#include <States/GameState/GameState.h>
#include <States/SelectState/SelectState.h>

void GameState::event_handling() {
    sf::Event event{};
    while (m_window.pollEvent(event)) {
        if (event.type == sf::Event::Closed) {
            m_state_manager.set_next_state(std::make_unique<SelectState>(m_state_manager, config::SELECT_LEVEL_VIDEO_MODE, config::SELECT_LEVEL_TITLE));
        }
        if (event.type == sf::Event::KeyPressed && event.key.code == sf::Keyboard::Z && event.key.control)
            m_context_manager.restore_previous_context();
        if (event.type == sf::Event::KeyPressed && m_context_manager.get_current_context().state == GameContext::INGAME)
            process_key_pressed(event.key.code);
    }
}

void GameState::update() {
    if (m_context_manager.get_current_context().state != GameContext::INGAME)
        return;
    std::vector<std::unique_ptr<IGameEvent>> events;

    for (const auto& obj : m_context_manager.get_current_context().dynamic_objects) {
        obj->action();
        if (auto event_ptr = obj->accept(&m_context_manager.get_current_context().pacman); event_ptr != nullptr)
            events.emplace_back(std::move(event_ptr));
    }

    for (const auto& obj : m_context_manager.get_current_context().static_objects) {
        if (auto event_ptr = obj->accept(&m_context_manager.get_current_context().pacman); event_ptr != nullptr)
            events.emplace_back(std::move(event_ptr));
    }

    if (m_context_manager.get_current_context().static_objects.empty()) {
        events.emplace_back(std::make_unique<WinGame>());
    }

    for (const auto& event : events)
        event->handle(m_context_manager.get_current_context());
}

void GameState::render() {
    switch (m_context_manager.get_current_context().state) {
        case GameContext::INGAME:
            m_window.clear(config::GAME_COLOR_BACKGROUND_INGAME);
            break;
        case GameContext::WIN:
            m_window.clear(config::GAME_COLOR_BACKGROUND_WIN);
            break;
        case GameContext::LOST:
            m_window.clear(config::GAME_COLOR_BACKGROUND_LOST);
    }
    m_maze.draw_into(m_window);
    for (const auto& obj : m_context_manager.get_current_context().static_objects)
        obj->draw_into(m_window);
    for (const auto& obj : m_context_manager.get_current_context().dynamic_objects)
        obj->draw_into(m_window);
    m_context_manager.get_current_context().pacman.draw_into(m_window);
    m_window.display();
}

void GameState::process_key_pressed(const sf::Keyboard::Key key) {
    switch (key) {
        case config::KEY_UP:
            m_context_manager.save_context();
            m_context_manager.get_current_context().pacman.move(Room::UP);
            break;
        case config::KEY_DOWN:
            m_context_manager.save_context();
            m_context_manager.get_current_context().pacman.move(Room::DOWN);
            break;
        case config::KEY_LEFT:
            m_context_manager.save_context();
            m_context_manager.get_current_context().pacman.move(Room::LEFT);
            break;
        case config::KEY_RIGHT:
            m_context_manager.save_context();
            m_context_manager.get_current_context().pacman.move(Room::RIGHT);
            break;
        default: break;
    }
}

bool GameState::do_step() {
    event_handling();
    update();
    render();
    return true;
}
